package org.bitbucket.bradleysmithllc.etlunit.feature.database.db;

import edu.emory.mathcs.backport.java.util.Collections;
import org.bitbucket.bradleysmithllc.etlunit.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class DatabaseImpl implements Database
{
	private final String name;

	private final List<Catalog> catalogList = new ArrayList<Catalog>();
	private final Map<String, Catalog> catalogMap = new HashMap<String, Catalog>();

	public DatabaseImpl(String name) {
		this.name = name;
	}

	@Override
	public String getName() {
		return name;
	}

	@Override
	public List<Catalog> getCatalogs() {
		return Collections.unmodifiableList(catalogList);
	}

	@Override
	public Map<String, Catalog> getCatalogMap() {
		return catalogMap;
	}

	@Override
	public Catalog getCatalog(String name) {
		String safeNullableUppercase = StringUtils.getSafeNullableUppercase(name);
		Catalog c = catalogMap.get(safeNullableUppercase);

		if (c == null)
		{
			return addCatalog(new CatalogImpl(this, safeNullableUppercase));
		}

		return c;
	}

	public Catalog addCatalog(Catalog ca)
	{
		String safeNullableUppercase = StringUtils.getSafeNullableUppercase(ca.getName());
		if (catalogMap.containsKey(safeNullableUppercase))
		{
			throw new IllegalArgumentException("Catalog with name [" + safeNullableUppercase + "] already added.");
		}

		catalogMap.put(safeNullableUppercase, ca);
		catalogList.add(ca);

		return ca;
	}

	@Override
	public String toString() {
		return name;
	}
}